import logging
from os import path, makedirs
from webbrowser import open as open_url

import requests
from click import style
from cloudinary import Search, uploader
from cloudinary.utils import cloudinary_url

from cloudinary_cli.defaults import logger
from cloudinary_cli.utils.json_utils import print_json, write_json_to_file
from cloudinary_cli.utils.utils import print_help, parse_args_kwargs, parse_option_value, log_exception


def query_cld_folder(folder):
    files = {}
    next_cursor = True
    expression = Search().expression(f"folder:{folder}/*").with_field("image_analysis").max_results(500)
    while next_cursor:
        res = expression.execute()

        for asset in res['resources']:
            rel_path = path.relpath(asset['public_id'], folder)
            files[rel_path] = {
                "type": asset['type'],
                "resource_type": asset['resource_type'],
                "public_id": asset['public_id'],
                "format": asset['format'],
                "etag": asset.get('etag', '0'),
                "relative_path": rel_path,  # save for inner use
            }
        # use := when switch to python 3.8
        next_cursor = res.get('next_cursor')
        expression.next_cursor(next_cursor)

    return files


def upload_file(file_path, options, uploaded=None, skipped=None):
    uploaded = uploaded if uploaded is not None else []
    skipped = skipped if skipped is not None else []
    verbose = logger.getEffectiveLevel() < logging.INFO

    try:
        result = uploader.upload(file_path, **options)
        logger.info(style(f"Successfully uploaded {file_path} as {result['public_id']}", fg="green"))
        if verbose:
            print_json(result)
        uploaded.append(result['public_id'])
    except Exception as e:
        log_exception(e, f"Failed uploading {file_path}")
        skipped.append(file_path)
        raise


def download_file(remote_file, local_path):
    makedirs(path.dirname(local_path), exist_ok=True)
    with open(local_path, "wb") as f:
        f.write(requests.get(cloudinary_url(remote_file['public_id'], resource_type=remote_file['resource_type'],
                                            type=remote_file['type'])[0]).content)
    logger.info(style("Downloaded '{}' to '{}'".format(remote_file['relative_path'], local_path), fg="green"))


def handle_api_command(
        params,
        optional_parameter,
        optional_parameter_parsed,
        ls,
        save,
        doc,
        doc_url,
        api_instance,
        api_name):
    """
    Used by Admin and Upload API commands
    """
    if doc:
        return open_url(doc_url)

    if ls or len(params) < 1:
        return print_help(api_instance)

    try:
        func = api_instance.__dict__[params[0]]
        if not callable(func):
            raise Exception(f"{func} is not callable")
    except Exception:
        raise Exception(f"Method {params[0]} does not exist in the {api_name.capitalize()} API.")

    args, kwargs = parse_args_kwargs(func, params[1:]) if len(params) > 1 else ([], {})

    res = func(*args, **{
        **kwargs,
        **{k: v for k, v in optional_parameter},
        **{k: parse_option_value(v) for k, v in optional_parameter_parsed},
    })

    print_json(res)

    if save:
        write_json_to_file(res, save)



#!/usr/bin/env python3
import os

import cloudinary
from cloudinary import api

from cloudinary_cli.defaults import CLOUDINARY_CLI_CONFIG_FILE, OLD_CLOUDINARY_CLI_CONFIG_FILE, logger
from cloudinary_cli.utils.json_utils import write_json_to_file, read_json_from_file
from cloudinary_cli.utils.utils import log_exception


def load_config():
    if not os.path.exists(CLOUDINARY_CLI_CONFIG_FILE) or os.path.getsize(CLOUDINARY_CLI_CONFIG_FILE) < 1:
        return {}

    return read_json_from_file(CLOUDINARY_CLI_CONFIG_FILE)


def save_config(config):
    _verify_file_path(CLOUDINARY_CLI_CONFIG_FILE)
    write_json_to_file(config, CLOUDINARY_CLI_CONFIG_FILE)


def update_config(new_config):
    curr_config = load_config()
    curr_config.update(new_config)
    save_config(curr_config)


def remove_config_keys(*keys):
    curr_config = load_config()
    not_found = []
    for key in keys:
        if not curr_config.pop(key, None):
            not_found.append(key)

    save_config(curr_config)

    return not_found


def refresh_cloudinary_config(cloudinary_url):
    os.environ.update({'CLOUDINARY_URL': cloudinary_url})
    cloudinary.reset_config()


def verify_cloudinary_url(cloudinary_url):
    refresh_cloudinary_config(cloudinary_url)
    try:
        api.ping()
    except Exception as e:
        log_exception(e, f"Invalid Cloudinary URL: {cloudinary_url}")
        return False
    return True


def migrate_old_config():
    """
    Migrate old config file (if exists) to new location
    """
    if not os.path.exists(OLD_CLOUDINARY_CLI_CONFIG_FILE):
        return

    try:
        old_config = read_json_from_file(OLD_CLOUDINARY_CLI_CONFIG_FILE)
    except Exception:
        logger.error(f"Unable to parse old Cloudinary config file: {OLD_CLOUDINARY_CLI_CONFIG_FILE}, "
                     f"please fix or remove it")
        raise

    new_config = load_config()
    new_config.update(old_config)
    save_config(new_config)

    os.remove(OLD_CLOUDINARY_CLI_CONFIG_FILE)


def initialize():
    migrate_old_config()


def _verify_file_path(file):
    os.makedirs(os.path.dirname(file), exist_ok=True)



import logging
from functools import reduce
from itertools import product
from os import sep, remove
from os.path import split, join as path_join, abspath

from click import command, argument, option, style
from cloudinary import api

from cloudinary_cli.utils.api_utils import query_cld_folder, upload_file, download_file
from cloudinary_cli.utils.file_utils import walk_dir, delete_empty_dirs
from cloudinary_cli.utils.json_utils import print_json
from cloudinary_cli.utils.utils import logger, run_tasks_concurrently, confirm_action

DELETE_ASSETS_BATCH_SIZE = 100


@command("sync",
         short_help="Synchronize between a local directory and a Cloudinary folder.",
         help="Synchronize between a local directory and a Cloudinary folder, maintaining the folder structure.")
@argument("local_folder")
@argument("cloudinary_folder")
@option("--push", help="Push changes from your local folder to your Cloudinary folder.", is_flag=True)
@option("--pull", help="Pull changes from your Cloudinary folder to your local folder.", is_flag=True)
@option("-w", "--concurrent_workers", type=int, default=30, help="Specify number of concurrent network threads.")
@option("-F", "--force", is_flag=True, help="Skip confirmation when deleting files.")
def sync(local_folder, cloudinary_folder, push, pull, concurrent_workers, force):
    if push == pull:
        raise Exception("Please use either the '--push' OR '--pull' options")

    sync_dir = SyncDir(local_folder, cloudinary_folder, concurrent_workers, force)

    if push:
        sync_dir.push()
    elif pull:
        sync_dir.pull()

    logger.info("Done!")


class SyncDir:
    def __init__(self, local_dir, remote_dir, concurrent_workers, force):
        self.local_dir = local_dir
        self.remote_dir = remote_dir
        self.concurrent_workers = concurrent_workers
        self.force = force

        self.verbose = logger.getEffectiveLevel() < logging.INFO

        self.local_files = walk_dir(abspath(self.local_dir))
        logger.info(f"Found {len(self.local_files)} items in local folder '{local_dir}'")

        self.remote_files = query_cld_folder(self.remote_dir)
        logger.info(f"Found {len(self.remote_files)} items in Cloudinary folder '{self.remote_dir}'")

        local_file_names = self.local_files.keys()
        remote_file_names = self.remote_files.keys()

        self.unique_remote_file_names = remote_file_names - local_file_names
        self.unique_local_file_names = local_file_names - remote_file_names
        common_file_names = local_file_names - self.unique_local_file_names

        self.out_of_sync_file_names = self._get_out_of_sync_file_names(common_file_names)

        skipping = len(common_file_names) - len(self.out_of_sync_file_names)

        if skipping:
            logger.info(f"Skipping {skipping} items")

    def _get_out_of_sync_file_names(self, common_file_names):
        logger.debug("\nCalculating differences...\n")
        out_of_sync_file_names = set()
        for f in common_file_names:
            if self.local_files[f]['etag'] != self.remote_files[f]['etag']:
                out_of_sync_file_names.add(f)
                continue
            logger.debug(f"{f} is in sync")

        return out_of_sync_file_names

    def push(self):
        if not self._delete_unique_remote_files():
            logger.info("Aborting...")
            return False

        files_to_push = self.unique_local_file_names | self.out_of_sync_file_names

        to_upload = list(filter(lambda x: split(x)[1][0] != ".", files_to_push))
        logger.info(f"Uploading {len(to_upload)} items to Cloudinary folder '{self.remote_dir}'")

        options = {
            'use_filename': True,
            'unique_filename': False,
            'invalidate': True,
            'resource_type': 'auto'
        }
        uploads = []
        for file in to_upload:
            folder = path_join(self.remote_dir, sep.join(file.split(sep)[:-1]))

            uploads.append((self.local_files[file]['path'], {**options, 'folder': folder}))

        run_tasks_concurrently(upload_file, uploads, self.concurrent_workers)

    def _delete_unique_remote_files(self):
        if not len(self.unique_remote_file_names):
            return True

        if not (self.force or confirm_action(
                f"Running this command will delete {len(self.unique_remote_file_names)} remote files. "
                f"Continue? (y/N)")):
            return False

        logger.info(f"Deleting {len(self.unique_remote_file_names)} resources "
                    f"from Cloudinary folder '{self.remote_dir}'")
        files_to_delete_from_cloudinary = list(map(lambda x: self.remote_files[x], self.unique_remote_file_names))

        for i in product({"upload", "private", "authenticated"}, {"image", "video", "raw"}):
            batch = list(map(lambda x: x['public_id'],
                             filter(lambda x: x["type"] == i[0] and x["resource_type"] == i[1],
                                    files_to_delete_from_cloudinary)))
            if not len(batch):
                continue

            logger.info("Deleting {} resources with type '{}' and resource_type '{}'".format(len(batch), *i))
            counter = 0
            while counter * DELETE_ASSETS_BATCH_SIZE < len(batch) and len(batch) > 0:
                counter += 1
                res = api.delete_resources(
                    batch[(counter - 1) * DELETE_ASSETS_BATCH_SIZE:counter * DELETE_ASSETS_BATCH_SIZE], invalidate=True,
                    resource_type=i[1], type=i[0])
                num_deleted = reduce(lambda x, y: x + 1 if y == "deleted" else x, res['deleted'].values(), 0)
                if self.verbose:
                    print_json(res)
                if num_deleted != len(batch):
                    logger.error("Failed deletes:\n{}".format("\n".join(list(
                        map(lambda x: x[0], filter(lambda x: x[1] != 'deleted', res['deleted'].items()))))))
                else:
                    logger.info(style(f"Deleted {num_deleted} resources", fg="green"))

        return True

    def pull(self):
        if not self._delete_unique_local_files():
            logger.info("Aborting...")
            return False

        if not (self.force or confirm_action(
                f"Running this command will delete {len(self.unique_local_file_names)} local files. "
                f"Continue? (y/N)")):
            return False

        files_to_pull = self.unique_remote_file_names | self.out_of_sync_file_names

        logger.info(f"Downloading {len(files_to_pull)} files from Cloudinary")

        downloads = []
        for file in files_to_pull:
            remote_file = self.remote_files[file]
            file_name = file + "." + remote_file['format'] if remote_file['resource_type'] != 'raw' else file
            local_path = abspath(path_join(self.local_dir, file_name))

            downloads.append((remote_file, local_path))

        run_tasks_concurrently(download_file, downloads, self.concurrent_workers)

    def _delete_unique_local_files(self):
        if not len(self.unique_local_file_names):
            return

        logger.info(f"Deleting {len(self.unique_local_file_names)} local files...")
        for file in self.unique_local_file_names:
            path = abspath(self.local_files[file]['path'])
            remove(path)
            logger.info(f"Deleted '{path}'")

        logger.info("Deleting empty folders...")
        delete_empty_dirs(self.local_dir)

        return True



import click

from cloudinary_cli.core.admin import admin
from cloudinary_cli.core.config import config
from cloudinary_cli.core.search import search
from cloudinary_cli.core.uploader import uploader
from cloudinary_cli.core.utils import url
from cloudinary_cli.core.overrides import resolve_command

setattr(click.MultiCommand, "resolve_command", resolve_command)

commands = [
    config,
    search,
    admin,
    uploader,
    url,
]



from cloudinary_cli.utils.utils import parse_option_value


def test_parse_option_value():
    """ should parse option values correctly """
    assert parse_option_value("haha,123,1234") == "haha,123,1234"
    assert parse_option_value("True") is True
    assert parse_option_value("false") is False
    assert parse_option_value('["test","123"]') == ['test', '123']
    assert parse_option_value('{"foo":"bar"}') == {"foo": "bar"}
    assert parse_option_value('{"an":"object","or":"dict"}') == {"an": "object", "or": "dict"}
    assert parse_option_value('["this","will","be","read","as","a","list"]') == ["this", "will", "be", "read", "as",
                                                                                 "a", "list"]


def test_parse_option_value_converts_int_to_str():
    """ should convert a parsed int to a str """
    assert parse_option_value(1) == "1"



from click.parser import split_opt
from click.utils import make_str
from cloudinary import api, uploader
from cloudinary.uploader import upload as original_upload


# overrides click.MultiCommand.resolve_command
def resolve_command(self, ctx, args):
    # Patch the `resolve_command` function to enable simple commands (eg. cld resource)
    # Only core commands from API and modules are registered (eg. cld admin)
    cmd_name = make_str(args[0])
    original_cmd_name = cmd_name

    cmd = self.get_command(ctx, cmd_name)
    if cmd is None and ctx.token_normalize_func is not None:
        cmd_name = ctx.token_normalize_func(cmd_name)
        cmd = self.get_command(ctx, cmd_name)

    if cmd is None and not ctx.resilient_parsing:
        if split_opt(cmd_name)[0]:
            self.parse_args(ctx, ctx.args)

        if original_cmd_name in api.__dict__:
            cmd = self.get_command(ctx, "admin")
            return cmd_name, cmd, args
        elif original_cmd_name in uploader.__dict__:
            cmd = self.get_command(ctx, "uploader")
            return cmd_name, cmd, args
        else:
            ctx.fail('No such command "%s".' % original_cmd_name)

    return cmd_name, cmd, args[1:]


# Patch to set `auto` resource type
def upload(file, **options):
    if "resource_type" not in options.keys():
        options["resource_type"] = "auto"
    return original_upload(file, **options)



import json
from platform import system

import click
from pygments import highlight, lexers, formatters


def write_json_to_file(json_obj, filename, indent=2, sort_keys=False):
    with open(filename, 'w') as file:
        json.dump(json_obj, file, indent=indent, sort_keys=sort_keys)


def read_json_from_file(filename):
    with open(filename, 'r') as file:
        return json.loads(file.read() or "{}")


def print_json(res):
    res_str = json.dumps(res, indent=2)

    if system() != "Windows":
        res_str = highlight(res_str.encode('UTF-8'), lexers.JsonLexer(), formatters.TerminalFormatter()).strip()

    click.echo(res_str)



from os import getcwd, walk
from os.path import dirname, split, join as path_join, abspath

from click import command, argument, option, style

from cloudinary_cli.utils.api_utils import upload_file
from cloudinary_cli.utils.utils import parse_option_value, logger, run_tasks_concurrently


@command("upload_dir", help="""Upload a folder of assets, maintaining the folder structure.""")
@argument("directory", default=".")
@option("-o", "--optional_parameter", multiple=True, nargs=2, help="Pass optional parameters as raw strings.")
@option("-O", "--optional_parameter_parsed",
        multiple=True,
        nargs=2,
        help="Pass optional parameters as interpreted strings.")
@option("-t", "--transformation", help="The transformation to apply on all uploads.")
@option("-f", "--folder", default="",
        help="The Cloudinary folder where you want to upload the assets. "
             "You can specify a whole path, for example folder1/folder2/folder3. "
             "Any folders that do not exist are automatically created.")
@option("-p", "--preset", help="The upload preset to use.")
@option("-w", "--concurrent_workers", type=int, default=30, help="Specify number of concurrent network threads.")
def upload_dir(directory, optional_parameter, optional_parameter_parsed, transformation, folder, preset,
               concurrent_workers):
    items, skipped = [], []
    dir_to_upload = abspath(path_join(getcwd(), directory))
    logger.info("Uploading directory '{}'".format(dir_to_upload))
    parent = dirname(dir_to_upload)
    options = {
        **{k: v for k, v in optional_parameter},
        **{k: parse_option_value(v) for k, v in optional_parameter_parsed},
        "resource_type": "auto",
        "invalidate": True,
        "unique_filename": False,
        "use_filename": True,
        "raw_transformation": transformation,
        "upload_preset": preset
    }

    uploads = []

    for root, _, files in walk(dir_to_upload):
        for fi in files:
            file_path = abspath(path_join(dir_to_upload, root, fi))
            mod_folder = path_join(folder, dirname(file_path[len(parent) + 1:]))
            if split(file_path)[1][0] == ".":
                continue
            options = {**options, "folder": mod_folder}
            uploads.append((file_path, options, items, skipped))

    run_tasks_concurrently(upload_file, uploads, concurrent_workers)

    logger.info(style("{} resources uploaded".format(len(items)), fg="green"))
    if skipped:
        logger.warn("{} items skipped".format(len(skipped)))



from webbrowser import open as open_url

import cloudinary
from click import command, argument, option

from cloudinary_cli.defaults import logger
from cloudinary_cli.utils.json_utils import write_json_to_file, print_json
from cloudinary_cli.utils.utils import write_json_list_to_csv, confirm_action

DEFAULT_MAX_RESULTS = 500


@command("search",
         short_help="Run the admin API search method.",
         help="""\b
Run the admin API search method.
Format: cld <cli options> search <command options> <Lucene query syntax string>
e.g. cld search cat AND tags:kitten -s public_id desc -f context -f tags -n 10
""")
@argument("query", nargs=-1)
@option("-f", "--with_field", multiple=True, help="Specify which asset attribute to include in the result.")
@option("-s", "--sort_by", nargs=2, help="Sort search results by (field, <asc|desc>).")
@option("-a", "--aggregate", nargs=1,
        help="Specify the attribute for which an aggregation count should be calculated and returned.")
@option("-n", "--max_results", nargs=1, default=10,
        help="The maximum number of results to return. Default: 10, maximum: 500.")
@option("-c", "--next_cursor", nargs=1, help="Continue a search using an existing cursor.")
@option("-A", "--auto_paginate", is_flag=True, help="Return all results. Will call Admin API multiple times.")
@option("-F", "--force", is_flag=True, help="Skip confirmation when running --auto-paginate.")
@option("-ff", "--filter_fields", multiple=True, help="Filter fields to return.")
@option("--json", nargs=1, help="Save JSON output to a file. Usage: --json <filename>")
@option("--csv", nargs=1, help="Save CSV output to a file. Usage: --csv <filename>")
@option("-d", "--doc", is_flag=True, help="Open Search API documentation page.")
def search(query, with_field, sort_by, aggregate, max_results, next_cursor,
           auto_paginate, force, filter_fields, json, csv, doc):
    if doc:
        return open_url("https://cloudinary.com/documentation/search_api")

    fields_to_keep = []
    if filter_fields:
        for f in list(filter_fields):
            if "," in f:
                fields_to_keep += f.split(",")
            else:
                fields_to_keep.append(f)
        fields_to_keep = tuple(fields_to_keep) + with_field

    expression = cloudinary.search.Search().expression(" ".join(query))

    if auto_paginate:
        max_results = DEFAULT_MAX_RESULTS
    if with_field:
        for f in with_field:
            expression.with_field(f)
    if sort_by:
        expression.sort_by(*sort_by)
    if aggregate:
        expression.aggregate(aggregate)
    if next_cursor:
        expression.next_cursor(next_cursor)

    expression.max_results(max_results)

    res = execute_single_request(expression, fields_to_keep)

    if auto_paginate:
        res = handle_auto_pagination(res, expression, force, fields_to_keep)

    print_json(res)

    if json:
        write_json_to_file(res['resources'], json)
        logger.info(f"Saved search JSON to '{json}' file")

    if csv:
        write_json_list_to_csv(res['resources'], csv, fields_to_keep)
        logger.info(f"Saved search to '{csv}.csv' file")


def execute_single_request(expression, fields_to_keep):
    res = expression.execute()

    if fields_to_keep:
        res['resources'] = list(
            map(lambda x: {k: x[k] if k in x.keys() else None for k in fields_to_keep}, res['resources'])
        )

    return res


def handle_auto_pagination(res, expression, force, fields_to_keep):
    if 'next_cursor' not in res.keys():
        return res

    if not force:
        if not confirm_action(
                f"{res['total_count']} total results. "
                f"{res.rate_limit_remaining + 1} Admin API rate limit remaining.\n"
                f"Running this query will use {res['total_count'] // DEFAULT_MAX_RESULTS + 1} Admin API calls. "
                f"Continue? (y/N)"):
            logger.info("Stopping. Please run again without -A.")
            return res
        else:
            logger.info("Continuing. You may use the -F flag to force auto_pagination.")

    all_results = res
    while 'next_cursor' in res.keys():
        expression.next_cursor(res['next_cursor'])

        res = execute_single_request(expression, fields_to_keep)

        all_results['resources'] += res['resources']
        all_results['time'] += res['time']

    all_results.pop('next_cursor')  # it is empty by now

    return all_results



import cloudinary
from click import command, option, echo

from cloudinary_cli.defaults import logger
from cloudinary_cli.utils.config_utils import load_config, verify_cloudinary_url, update_config, remove_config_keys


@command("config", help="Display the current configuration, and manage additional configurations.")
@option("-n", "--new", help="""\b Create and name a configuration from a Cloudinary account environment variable.
e.g. cld config -n <NAME> <CLOUDINARY_URL>""", nargs=2)
@option("-ls", "--ls", help="List all saved configurations.", is_flag=True)
@option("-rm", "--rm", help="Delete a specified configuration.", nargs=1)
@option("-url", "--from_url",
        help="Create a configuration from a Cloudinary account environment variable. "
             "The configuration name is the cloud name.",
        nargs=1)
def config(new, ls, rm, from_url):
    if new or from_url:
        config_name, cloudinary_url = new or [None, from_url]

        if not verify_cloudinary_url(cloudinary_url):
            return

        config_name = config_name or cloudinary.config().cloud_name

        update_config({config_name: cloudinary_url})

        logger.info("Config '{}' saved!".format(config_name))
        logger.info("Example usage: cld -C {} <command>".format(config_name))
    elif rm:
        if remove_config_keys(rm):
            logger.warn(f"Configuration '{rm}' not found.")
        else:
            logger.info(f"Configuration '{rm}' deleted")
    elif ls:
        echo("\n".join(load_config().keys()))
    else:
        obfuscated_config = {k: v if k != "api_secret" else "***************{}".format(v[-4:])
                             for k, v in cloudinary.config().__dict__.items()}
        echo('\n'.join(["{}:\t{}".format(k, v) for k, v in obfuscated_config.items()]))



from webbrowser import open as open_url

from click import command, argument, option, Choice, echo
from cloudinary.utils import cloudinary_url


@command("url", help="Generate a Cloudinary URL, which you can optionally open in your browser.")
@argument("public_id", required=True)
@argument("transformation", default="")
@option("-rt", "--resource_type", default="image", type=Choice(['image', 'video', 'raw']), help="The asset type")
@option("-t", "--type", "delivery_type", default="upload",
        type=Choice(['upload', 'private', 'authenticated', 'fetch', 'list', 'url2png']),
        help="The delivery type.")
@option("-o", "--open", 'open_in_browser', is_flag=True, help="Generate the derived asset and open it in your browser.")
@option("-s", "--sign", is_flag=True, help="Generate a signed URL.", default=False)
def url(public_id, transformation, resource_type, delivery_type, open_in_browser, sign):
    if delivery_type == "authenticated" or resource_type == "url2png":
        sign = True
    elif delivery_type == "list":
        public_id += ".json"

    res = cloudinary_url(public_id, resource_type=resource_type,
                         raw_transformation=transformation, type=type, sign_url=sign)[0]
    echo(res)

    if open_in_browser:
        open_url(res)



#!/usr/bin/env python3

import json
import os
from csv import DictWriter
from functools import reduce
from hashlib import md5
from inspect import signature, getfullargspec
from multiprocessing import pool

import cloudinary
from jinja2 import Environment

from cloudinary_cli.defaults import logger, TEMPLATE_FOLDER

not_callable = ['is_appengine_sandbox', 'call_tags_api', 'call_context_api', 'call_cacheable_api', 'call_api', 'text']

BLOCK_SIZE = 65536


def etag(fi):
    file_hash = md5()
    with open(fi, 'rb') as f:
        fb = f.read(BLOCK_SIZE)
        while len(fb) > 0:
            file_hash.update(fb)
            fb = f.read(BLOCK_SIZE)

    return file_hash.hexdigest()


def get_help_str(api):
    funcs = list(filter(lambda x: callable(api.__dict__[x]) and x[0].islower() and x not in not_callable,
                        api.__dict__.keys()))
    return '\n'.join(["{0:25}({1})".format(x, ", ".join(list(signature(api.__dict__[x]).parameters))) for x in funcs])


def print_help(api):
    logger.info(get_help_str(api))


def log_exception(e, message=None):
    message = f"{message}, error: {str(e)}" if message is not None else str(e)
    logger.debug(message, exc_info=True)
    logger.error(message)


def load_template(language, template_name):
    filepath = os.path.join(TEMPLATE_FOLDER, language, template_name)
    try:
        with open(filepath) as f:
            template = Environment().from_string(f.read())
    except IOError:
        logger.error(f"Failed loading template: '{template_name}' for language: '{language}'")
        raise
    return template.render(**cloudinary.config().__dict__)


def parse_option_value(value):
    if value == "True" or value == "true":
        return True
    elif value == "False" or value == "false":
        return False
    try:
        value = json.loads(value)
    except Exception:
        pass
    if isinstance(value, int):
        value = str(value)
    return value


def parse_args_kwargs(func, params):
    spec = getfullargspec(func)
    n_args = len(spec.args) if spec.args else 0
    n_defaults = len(spec.defaults) if spec.defaults else 0

    n_req = n_args - n_defaults
    if len(params) < n_req:
        raise Exception("Function '{}' requires {} arguments".format(func.__name__, n_req))
    args = [parse_option_value(x) for x in params[:n_req]]

    kwargs = {k: parse_option_value(v) for k, v in [x.split('=') for x in params[n_req:]]} if params[n_req:] else {}
    return args, kwargs


def remove_string_prefix(string, prefix):
    return string[string.startswith(prefix) and len(prefix):]


def write_json_list_to_csv(json_list, filename, fields_to_keep=()):
    with open(f'{filename}.csv', 'w') as f:
        if not fields_to_keep:
            fields_to_keep = list(reduce(lambda x, y: set(y.keys()) | x, json_list, set()))

        writer = DictWriter(f, fieldnames=fields_to_keep)
        writer.writeheader()
        writer.writerows(json_list)


def run_tasks_concurrently(func, tasks, concurrent_workers):
    thread_pool = pool.ThreadPool(concurrent_workers)
    thread_pool.starmap(func, tasks)


def confirm_action(message):
    r = input(message)
    if r.lower() != 'y':
        return False

    return True



from os import walk, path, listdir, rmdir

from cloudinary_cli.defaults import logger
from cloudinary_cli.utils.utils import etag


def walk_dir(root_dir):
    all_files = {}
    for root, _, files in walk(root_dir):
        for file in files:
            all_files[path.splitext(path.join(root, file)[len(root_dir) + 1:])[0]] = {
                "path": path.join(root, file),
                "etag": etag(path.join(root, file))
            }
    return all_files


def delete_empty_dirs(root, remove_root=False):
    if not path.isdir(root):
        return

    files = listdir(root)
    if len(files):
        for f in files:
            full_path = path.join(root, f)
            if path.isdir(full_path):
                delete_empty_dirs(full_path, True)

    files = listdir(root)
    if len(files) == 0 and remove_root:
        logger.debug(f"Removing empty folder '{root}'")
        rmdir(root)



from os.path import join

from click import command, argument, option
from cloudinary import api
from cloudinary.utils import cloudinary_url
from requests import head

from cloudinary_cli.utils.utils import logger


@command("migrate",
         short_help="Migrate files using an existing auto-upload mapping and a file of URLs.",
         help="Migrate a list of external media files to Cloudinary. "
              "The URLs of the files to migrate are listed in a separate file and must all have the same prefix.")
@argument("upload_mapping")
@argument("file")
@option("-d", "--delimiter", default="\n", help="The separator used between the URLs. Default: New line")
@option("-v", "--verbose", is_flag=True)
def migrate(upload_mapping, file, delimiter, verbose):
    with open(file) as f:
        items = f.read().split(delimiter)
    mapping = api.upload_mapping(upload_mapping)
    items = map(lambda x: cloudinary_url(join(mapping['folder'], x[len(mapping['template']):])),
                filter(lambda x: x != '', items))
    for i in items:
        res = head(i[0])
        if res.status_code != 200:
            logger.error("Failed uploading asset: " + res.__dict__['headers']['X-Cld-Error'])
        elif verbose:
            logger.info("Uploaded {}".format(i[0]))



from webbrowser import open as open_url

import cloudinary
from click import command, argument, option
from cloudinary.utils import cloudinary_url


@command("sample", help="Open sample flowers image", hidden=True)
@argument("transformation", default="")
@option("-o", "--open", 'open_in_browser', is_flag=True, help="Open URL in your browser", )
def sample(transformation, open_in_browser):
    return _handle_sample_command("sample", transformation, open_in_browser)


@command("couple", help="Open sample couple image", hidden=True)
@argument("transformation", default="")
@option("-o", "--open", 'open_in_browser', is_flag=True, help="Open URL in your browser")
def couple(transformation, open_in_browser):
    return _handle_sample_command("couple", transformation, open_in_browser)


@command("dog", help="Open sample dog video", hidden=True)
@argument("transformation", default="")
@option("-o", "--open", 'open_in_browser', is_flag=True, help="Open URL in your browser")
def dog(transformation, open_in_browser):
    return _handle_sample_command("dog", transformation, open_in_browser, "video")


def _handle_sample_command(source, transformation=None, open_in_browser=False, resource_type="image"):
    cloudinary.config(cloud_name="demo", secure_distribution=None, cname=None)
    res = cloudinary_url(source, raw_transformation=transformation, resource_type=resource_type)[0]
    print(res)
    if open_in_browser:
        open_url(res)


commands = [
    sample,
    couple,
    dog,
]



import logging
import os
from os.path import join as path_join, expanduser, abspath, dirname

import click_log

logger = logging.getLogger(__name__)

click_log.basic_config(logger)

TEMPLATE_EXTS = {
    "python": "py",
    "html": "html",
    "ruby": "rb",
    "node": "js",
    "php": "php",
    "java": "java",
}

CLOUDINARY_HOME = os.environ.get('CLOUDINARY_HOME')

if CLOUDINARY_HOME is None:
    CLOUDINARY_HOME = abspath(path_join(expanduser("~"), ".cloudinary-cli"))

CLOUDINARY_CLI_CONFIG_FILE = abspath(path_join(CLOUDINARY_HOME, 'config.json'))

TEMPLATE_FOLDER_NAME = 'templates'
CLOUDINARY_CLI_ROOT = dirname(__file__)
TEMPLATE_FOLDER = path_join(CLOUDINARY_CLI_ROOT, TEMPLATE_FOLDER_NAME)
CUSTOM_TEMPLATE_FOLDER = path_join(abspath(CLOUDINARY_HOME), TEMPLATE_FOLDER_NAME)

OLD_CLOUDINARY_CLI_CONFIG_FILE = path_join(expanduser("~"), '.cloudinary-cli-config')



from click import command, argument, option
from cloudinary import uploader

from cloudinary_cli.core.overrides import upload
from cloudinary_cli.utils.api_utils import handle_api_command

uploader.upload = upload


@command("uploader",
         short_help="Run any methods that can be called through the upload API.",
         help="""
\b
Run any methods that can be called through the upload API.
Format: cld <cli options> uploader <command options> <method> <method parameters>
\te.g. cld uploader upload http://res.cloudinary.com/demo/image/upload/sample public_id=flowers invalidate=True
\b
\te.g. cld uploader rename flowers secret_flowers to_type=private
\t      OR
\t    cld uploader rename flowers secret_flowers -o to_type private
""")
@argument("params", nargs=-1)
@option("-o", "--optional_parameter", multiple=True, nargs=2, help="Pass optional parameters as raw strings.")
@option("-O", "--optional_parameter_parsed", multiple=True, nargs=2,
        help="Pass optional parameters as interpreted strings.")
@option("-ls", "--ls", is_flag=True, help="List all available methods in the Upload API.")
@option("--save", nargs=1, help="Save output to a file.")
@option("-d", "--doc", is_flag=True, help="Open the Upload API reference in a browser.")
def uploader(params, optional_parameter, optional_parameter_parsed, ls, save, doc):
    return handle_api_command(params, optional_parameter, optional_parameter_parsed, ls, save, doc,
                              doc_url="https://cloudinary.com/documentation/image_upload_api_reference",
                              api_instance=uploader,
                              api_name="upload")



#!/usr/bin/env python3

import setuptools
from cloudinary_cli import __version__


with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="cloudinary-cli",
    version=__version__,
    author="Cloudinary, Brian Luk",
    author_email="info@cloudinary.com, lukitsbrian@gmail.com",
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    description="A command line interface for Cloudinary with full API support",
    entry_points={
        'console_scripts': [
            'cld=cloudinary_cli.cli:main',
            'cloudinary=cloudinary_cli.cli:main',
            ],
    },
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cloudinary/cloudinary-cli",
    packages=setuptools.find_packages(),
    keywords='cloudinary cli pycloudinary image video digital asset management command line interface transformation '
             'friendly easy flexible',
    license="MIT",
    python_requires='>=3.6.0',
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    install_requires=[
        "cloudinary",
        "pygments",
        "jinja2",
        "click",
        "click-log",
        "requests"
    ],
    include_package_data=True,
    zip_safe=False
)



from click import command, argument, option
from cloudinary import api

from cloudinary_cli.utils.api_utils import handle_api_command


@command("admin",
         short_help="Run any methods that can be called through the admin API.",
         help="""\b
Run any methods that can be called through the admin API.
Format: cld <cli options> admin <command options> <method> <method parameters>
\te.g. cld admin resources max_results=10 tags=sample
\t      OR
\t    cld admin resources -o max_results 10 -o tags sample
\t      OR
\t    cld admin resources max_results=10 -o tags sample
""")
@argument("params", nargs=-1)
@option("-o", "--optional_parameter", multiple=True, nargs=2, help="Pass optional parameters as raw strings.")
@option("-O", "--optional_parameter_parsed", multiple=True, nargs=2,
        help="Pass optional parameters as interpreted strings.")
@option("-ls", "--ls", is_flag=True, help="List all available methods in the Admin API.")
@option("--save", nargs=1, help="Save output to a file.")
@option("-d", "--doc", is_flag=True, help="Open the Admin API reference in a browser.")
def admin(params, optional_parameter, optional_parameter_parsed, ls, save, doc):
    return handle_api_command(params, optional_parameter, optional_parameter_parsed, ls, save, doc,
                              doc_url="https://cloudinary.com/documentation/admin_api",
                              api_instance=api,
                              api_name="admin")



#!/usr/bin/env python3

import click
import click_log
import cloudinary

import cloudinary_cli.core
import cloudinary_cli.modules
import cloudinary_cli.samples
from cloudinary_cli.defaults import logger
from cloudinary_cli.utils.config_utils import initialize, load_config, refresh_cloudinary_config
from cloudinary_cli.utils.utils import log_exception

CONTEXT_SETTINGS = dict(max_content_width=click.get_terminal_size()[0], terminal_width=click.get_terminal_size()[0])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.option("-c", "--config",
              help="""Tell the CLI which account to run the command on by specifying an account environment variable."""
              )
@click.option("-C", "--config_saved",
              help="""Tell the CLI which account to run the command on by specifying a saved configuration - see 
              `config` command.""")
@click_log.simple_verbosity_option(logger)
def cli(config, config_saved):
    if config:
        refresh_cloudinary_config(config)
    elif config_saved:
        config = load_config()
        if config_saved not in config:
            raise Exception(f"Config {config_saved} does not exist")

        refresh_cloudinary_config(config[config_saved])

    if cloudinary.config().cloud_name is None:
        logger.warning("No Cloudinary configuration found.")

    return 0


def import_commands(*command_modules):
    for command_module in command_modules:
        for command in command_module:
            cli.add_command(command)


import_commands(
    cloudinary_cli.core.commands,
    cloudinary_cli.modules.commands,
    cloudinary_cli.samples.commands,
)


def main():
    initialize()
    try:
        exit_status = cli()
    except Exception as e:
        log_exception(e, "Command execution failed")
        exit_status = 1

    return exit_status


if __name__ == "__main__":
    main()



from click import command, argument, echo

from cloudinary_cli.defaults import TEMPLATE_EXTS
from cloudinary_cli.utils.utils import load_template


@command("make", short_help="Return template code for implementing the specified Cloudinary widget.",
         help="""\b
Return template code for implementing the specified Cloudinary widget.
e.g. cld make product_gallery
""")
@argument("template", nargs=-1)
def make(template):
    language = "html"
    if template[-1] in TEMPLATE_EXTS.keys():
        language = template[-1]
        template = template[:-1]
    elif template[0] in TEMPLATE_EXTS.keys():
        language = template[0]
        template = template[1:]

    echo(load_template(language, '_'.join(template)))
